#import "fontawesome.typ": github, fa

#let DEFAULT_STRONG = 300;
#let SMALL_STRONG = 200;

#let TITLE_FONT = "Bitter"
#let HEADING_FONT = TITLE_FONT
#let BODY_FONT = "Open Sans"

#let BODY_FONT_SIZE = 10pt
#let HEADING_FONT_SIZE = 1em
#let TITLE_FONT_SIZE = 2.25em

#let HEADING_LINE_GAP = 3mm
#let BODY_LINE_HEIGHT = 0.85em

// rest = not top
#let PAGE_MARGINS = (rest: 0.5in, top: 0.4in)

// Format locations.
#let format_location(location) = {
  [_#block(above: 0.7em, location)_]
}

#let indent(content) = {
  block(inset: (left: 1em, right: 0.5em), content)
}

// General entry that is split into a left and right half (for experience and education).
#let cv_entry(left_content: none, right_content: none, details: none) = {
  stack(
    dir: ttb,
    spacing: 0.9em,
    grid(columns: (11fr, 6fr), column-gutter: 1cm, {
      set strong(delta: DEFAULT_STRONG)
      set align(left)
      left_content
    }, {
      set strong(delta: SMALL_STRONG)
      set align(right)
      right_content
    }),
    {
      set align(left)
      set strong(delta: SMALL_STRONG)
      details
    },
  )
}

// Entry for work.
#let work_entry(
  role,
  company,
  tasks: none,
  start_date: none,
  end_date: none,
  location: none,
) = {
  let task_list = if tasks != none {
    let list = []
    for task in tasks {
      list += [- #task]
    }
    list
  } else {
    none
  }
  cv_entry(left_content: {
    set par(leading: 0.75em)
    text(1.1em)[*#role*]
    "\n"
    set strong(delta: SMALL_STRONG)
    text[*#company*]
  }, right_content: {
    [*#start_date -- #end_date*]

    if location != none {
      "\n" + format_location(location)
    }
  }, details: task_list)
}

#let project(title, repo_link, tools) = {
  set strong(delta: DEFAULT_STRONG)
  if repo_link != none {
    link(repo_link)[#text(1.1em)[*#title* #fa(github)]] + [ | _ #tools _ ]
  } else {
    text(1.1em)[*#title*] + [ | _ #tools _ ]
  }
}

// Set name and contact data and format headings
#let template(name, contact_data, color, doc) = {
  set page(margin: PAGE_MARGINS, paper: "us-letter")
  set list(indent: 1.25em, marker: [•])
  set text(font: (BODY_FONT), BODY_FONT_SIZE)
  set par(justify: true, leading: BODY_LINE_HEIGHT)
  align(center)[
    #text(size: TITLE_FONT_SIZE, font: TITLE_FONT, fill: color)[*#name*]
    #block(above: 0em, below: 1em)
    #{
      if contact_data != none and contact_data.len() > 0 {
        let elements = for el in contact_data {
          (link(el.link)[#{ el.service + " " + el.display }],)
        }
        text(1em)[#elements.join(" | ")]
      }
    }
  ]

  show heading.where(level: 1): i => {
    set align(left + horizon)
    let title = smallcaps(i.body)

    let colored_line(header_text) = style(styles => {
      let size = measure(header_text, styles)

      // 100% - spacing - width of text
      line(
        length: 100% - HEADING_LINE_GAP - size.width,
        stroke: (paint: color, thickness: 2pt, cap: "round"),
      )
    })

    set block(above: 0.9em, below: 0.75em)
    set text(size: 1em, fill: color, font: HEADING_FONT)
    stack(
      dir: ltr,
      spacing: HEADING_LINE_GAP,
      title,
      box(height: 2pt, fill: color, colored_line(title)),
    )
  }
  doc
}
